function [delay,G2,IF,IP] = G2main(I,dt,dpl)
% ---
% --- G2main calculate the intensity-intensity autocorrelation function
% --- in a multi-tau way using the symmetric normalization
% ---
% --- I needs to be a 2-D tensor or a row vector : [xpixel,times]
% ---
% ---    - only I is mandatory as a parameter
% ---    - the other parameter are optional and the routine can use 
% ---      standard values, however the 'delay' vector won't be correct
% ---
% --- Michael Sprung
% --- Rev 1.0 $ 2006/11/29 $ based on 'SNG2'
% --- 


% =========================================================================
% --- get frame dimension from input tensor
% =========================================================================
npixel   = size(I,1)                                                       ; % get the number of pixels
nframes  = size(I,2)                                                       ; % get the number of frames
% =========================================================================
% --- end of get frame dimension from input tensor
% =========================================================================


% =========================================================================
% --- check input parameter
% =========================================================================
if (nargin <= 2)
    dpl = 4                                                                ; % use default dpl
end                                              
if (nargin <= 1)
    dt  = 1                                                                ; % use default dt
end
% =========================================================================
% --- end of check input parameter
% =========================================================================


% =========================================================================
% --- create delays and time vector
% =========================================================================
totaldelays = finddelays(nframes,dpl,1)                                    ;
ntotaldelay = length(totaldelays)                                          ;
delay       = totaldelays * dt                                             ;
% =========================================================================
% --- end of creating delays and time vector
% =========================================================================


% =========================================================================
% --- initialize output tensors G2, IF & IP
% =========================================================================
G2 = zeros(npixel,ntotaldelay,'single')                                    ; % initialize G2
IF = zeros(npixel,ntotaldelay,'single')                                    ; % initialize IF (intensities fututre)
IP = zeros(npixel,ntotaldelay,'single')                                    ; % initialize IP (intensities past)
% =========================================================================
% --- end of initializing output tensors G2, IF & IP
% =========================================================================


% =========================================================================
% --- main loop
% =========================================================================
oldlevel   = 0                                                             ;
for k = 1 : ntotaldelay                                                      % loop over all delays

    % =====================================================================
    % --- create delay, level & tau
    % =====================================================================
    thisdelay = totaldelays(k)                                             ;
    level     = levelofdelay(thisdelay, dpl)                               ; % find the level of the actual delay
    if (level < 2)
        tau = thisdelay                                                    ;
    else
        tau = int32(thisdelay / 2.^(level-1) )                             ; % due to the ongoing data averaging the tau is not equal to the delay
    end
    % =====================================================================
    % --- end of creating delay, level & tau
    % =====================================================================

    
    % =====================================================================
    % --- frame calculation routine
    % =====================================================================
    if (level > oldlevel && level > 1)                                       % check: if the new delay is on a higher level than the data tensor can be averaged
        if ( mod(size(I,2),2) == 0)                                          % check if the number of frames is even or odd
            lasteven = size(I,2)                                           ;
            lastodd  = lasteven -1                                         ;
        else
            lasteven = size(I,2) - 1                                       ;
            lastodd  = lasteven -1                                         ;
        end
        % ---
        I(:,1:lasteven/2)     = (I(:,1:2:lastodd)+I(:,2:2:lasteven)) / 2   ; % average the data tensor
        I(:,lasteven/2+1:end) = []                                         ; % clear the useless part of the tensor
        clear lasteven lastodd                                             ;
    end
    % ---
    t         = 1 : (size(I,2)-tau)                                        ; % create time vector
    G2(:,k)   = mean(I(:,t) .* I(:,t+tau),2)                               ; % calculate: <I(q,t) * I(q,t+tau)>_t
    IP(:,k)   = mean(I(:,t    ),2)                                         ; % calculate: <I(q,t)>_t_left
    IF(:,k)   = mean(I(:,t+tau),2)                                         ; % calculate: <I(q,t+tau)>_t_right
    oldlevel  = level                                                      ; % redefine old level
    % =====================================================================
    % --- end of frame calculation routine
    % =====================================================================

end
clear I k level oldlevel t tau thisdelay                                   ;
clear dpl ntotaldelay totaldelays nframes                                  ;
% =========================================================================
% --- end of main loop
% =========================================================================


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;
% =========================================================================
% --- end of subfunction finddelays
% =========================================================================


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;
% =========================================================================
% --- end of subfunction indexofdelay
% =========================================================================


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;
% =========================================================================
% --- end of subfunction levelofdelay
% =========================================================================


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;
% =========================================================================
% --- end of subfunction delayofindex
% =========================================================================


% ---
% EOF
