function [psigma,covp,corp,r2,rv] = errorsigma(x,y,e,p,dp,free,func,f)
% =========================================================================
% --- ERRORSIGMA computes the sigma for each fitting parameter value
% --- call this function after a fit is performed
% --- [psigma,covp,corp,r2,rv] = errorsigma(x,y,e,p,dp,free,func,f)
% --- free should be 0 for fixed parameters, 1 for variable parameters
% --- a good choice for dp = 0.05 * p
% =========================================================================

    % =====================================================================
    % --- input preparation
    % =====================================================================
    x            = x(:)                                                    ; % x data
    y            = y(:)                                                    ; % y data
    e            = e(:)                                                    ; % error of data
    % ---
    selected     = find(e)                                                 ;
    wt           = zeros(size(y))                                          ; % create weighting vector
    wt(selected) = 1 ./ e(selected)                                        ;
    wt           = wt(:)                                                   ;
    % ---
    m            = length(x)                                               ; % number of data points
    p            = p(:)                                                    ; % parameters of final fit
    dp           = dp(:)                                                   ; % a vector telling how much to vary the parameter
    free         = free(:)                                                 ; % vector showing which parameters are varying

    % =====================================================================
    % --- check if best fit is supplied
    % =====================================================================
    if ( nargin <= 8 ) 
        f = []                                                             ;
    end
    if ( isempty(f) || (length(f) ~= m) )
        f = feval(func,p,x)                                                ;
    end
    f = f(:)                                                               ;

    % =====================================================================
    % --- check if all points are valid and use only valid points
    % =====================================================================
    index = find( ~isnan(x) & ~isnan(y) & ~isnan(wt)& ~isnan(f)         ...
                & ~isinf(f) & ~isinf(y) & ~isinf(wt) & wt)                 ;
    x  = x(index)                                                          ;
    y  = y(index)                                                          ;
    wt = wt(index)                                                         ;
    f  = f(index)                                                          ;
    m  = length(y)                                                         ;
    dp = dp .* free                                                        ;
    
    % =====================================================================
    % --- create dummy output parameter if early extit
    % =====================================================================
    psigma = []                                                            ;
    covp   = []                                                            ;
    corp   = []                                                            ;
    r2     = []                                                            ;
    rv     = []                                                            ;
    if ( isempty(p) == 1 )
        return
    end

    % =====================================================================
    % --- data reduction for data sets above 1000 points
    % =====================================================================
    nreduce = 1000                                                         ;
    if ( m > nreduce )                                                       % try to avoid crash if too many points
        ysav = y                                                           ;
        fsav = f                                                           ;
        msav = m                                                           ; % store the number of good data points
        wsav = wt                                                          ;
        % ---
        k    = m/nreduce                                                   ;
        k    = floor(k*(1:nreduce))                                        ;
        k    = k(k >= 1 & k <= m)                                          ;
        % ---
        x    = x(k)                                                        ;
        y    = y(k)                                                        ;
        f    = f(k)                                                        ;
        wt   = wt(k)                                                       ;
        m    = nreduce                                                     ;
    end

    % =====================================================================
    % --- calculate variance COV matrix & correlation matrix of parameters
    % --- & reevaluate the Jacobian at optimal values
    % --- The following section is Ray Muzic's estimate for covariance 
    % --- and correlation
    % --- covariance matrix of data estimate is from 
    % --- Bard Equation 7-5-13 and Row 1 of Table 5.1
    % =====================================================================

    % =====================================================================
    % --- calculate Jacobian
    % =====================================================================
    jac = partialderivatives(x,f,p,dp,func)                                ;
    msk = find(dp ~= 0)                                                    ; % use only 'free' parameter
    n   = length(msk)                                                      ; % reduce n to equal number of free parameters
    jac = jac(:, msk)                                                      ; % use only fitted parameters

    % =====================================================================
    % --- calculate squared inverse Jacobian
    % =====================================================================
    resid   = y-f                                                          ; %unweighted residuals
    Qinv    = diag(wt.*wt)                                                 ;
    jtgjinv = pinv(jac'*Qinv*jac)                                          ;
    if ( ~isempty(jtgjinv) )
        covp  = resid' * Qinv * resid / (m-n) * jtgjinv                    ;
        corp  = ones(n,n)                                                  ;
        d     = sqrt(abs(diag(covp)))                                      ;
        d2    = d*d'                                                       ;
        for j1 = 1 : n
            for j2 = 1 : n
                if ( d2(j1,j2) ~= 0 )
                    corp(j1,j2) = covp(j1,j2) / d2(j1,j2)                  ;
                end
            end
        end
    else
        disp('Warning : Could not compute squared inverse Jacobian!')      ;
        disp('Warning : No covariance and no correlation matirx!')         ;
        covp = zeros(n,n)                                                  ;
        corp =  ones(n,n)                                                  ;
    end
    stdp = sqrt(diag(covp))                                                ;

    % =====================================================================
    % --- calculate the error on the fitting parameter
    % =====================================================================
    j   = 1                                                                ;
    psigma = zeros(size(p))                                                ;
    for i = 1 : length(stdp)
        while free(j)==0
            j=j+1                                                          ;
        end
        psigma(j) = stdp(i)                                                ;
        j = j + 1                                                          ;
    end

    % =====================================================================
    % --- restore the data & calculate correlation coefficients ...
    % =====================================================================
    if ( nargout > 3 )
        if ( msav > nreduce )                                                      
            y  = ysav                                                      ;
            f  = fsav                                                      ;
            wt = wsav                                                      ;
        end
        r  = corrcoef(y.*wt,f.*wt)                                         ;
        r2 = r.*r'                                                         ;
        rv = sum(((f-y).*wt).^2/length(y))                                 ;
        clear r ysav fsav msav wsav                                        ;
    end
    clear nreduce                                                          ;

    % =====================================================================
    % --- Begin of subfunction partialderivatives
    % =====================================================================
    function df_dp = partialderivatives(x,f,p,dp,func)
    % =====================================================================
    % --- y = partialderivatives(x,y,p,dp,func)
    % --- Returns the partial derivatives of function 'func'.
    % --- 'x'(vect) is x axis values, 'y' is y values,
    % --- 'p' and 'dp' are parameters and their variation.
    % --- 'func' is the function concerned [y=func(x,p)]
    % --- Output 'df_dp' is a vector or matrix of partials varying of 'dp'
    % =====================================================================
        [nr,nc] = size(x)                                                  ;
        if ( nr < nc )
            x = x'                                                         ; 
        end
        p0    = p                                                          ; % save init params
        df_dp = zeros(length(x), length(p))                                ;
        i0 = find((abs(dp) <= 1e-12) | ~isfinite(dp) )                     ; % these parameters should not vary
        if ( ~isempty(i0) )
            dp(i0) = 0                                                     ;
        end
        for ii = 1 : length(p)
            if ( dp(ii) ~= 0 )
                p(ii) = p(ii) + dp(ii)                                     ;
                t = feval(func,p,x)                                        ;
                if ( length(t) ~= length(f) )
                    t = t(1:length(x))                                     ;
                end
                df_dp(:,ii) = (t(:)-f(:)) / dp(ii)                         ;
                p = p0                                                     ;
            end
        end
        if (nr < nc)
            df_dp = df_dp'                                                 ;
        end
    end
    % =====================================================================
    % --- End of subfunction partialderivatives
    % =====================================================================
    
end


% ---
% EOF
