function [varargout]=savefigure(fname,varargin)
% Easy way of saving figures to publication friendly format
%
% Usage: [Args=]savefigure(name[,property,value,...])
%
% properties and default value
% 'Dpi',300,
% 'Size',[3.3 2.5]      %inches
% 'FontSize',0          %fontsize multiplier
% 'MinFontSize',8       %minimum fontsize in pt
% 'LineWidth',.5        %linewidth multiplier
% 'MinLineWidth',.5     %minimum linewidth in pt
% 'MarkerSize',.5       %markersize multiplier
% 'MinMarkerSize',3     %minimum MarkerSize in pt
% 'Margins','auto'      %[Left,Right,Top,Bottom] - can be used to set margins (relative units)
% .                     %or a single multiplier to the tight-inset used for auto margins.
% 'PrintOptions','-dpng'
%
% AGU journals: Figures can be 20 picas wide or 41 picas wide and 58 picas deep.  That's
% 3.3 inches, 6.83, and 9.66. 
%
% (C) Aslak Grinsted 2002-2004

global sf_UndoStack
sf_UndoStack={};

CmPrInch=2.54;
f=gcf;


Args=struct('Dpi',300,...
    'Size',[3.3 2.5],  ...    %inches
    'FontSize',0,      ...    %fontsize multiplier
    'Margins','auto',...  %Left,Right,Top,Bottom - can be used to set margins (relative units)
    'LineWidth',.5,    ...    %linewidth multiplier
    'MarkerSize',.5,   ...    %markersize multiplier
    'MinFontSize',8,   ...    %minimum fontsize in pt
    'MinLineWidth',.5, ...    %minimum linewidth in pt
    'MinMarkerSize',3,  ...    %minimum MarkerSize in pt
    'BlackWhite',0, ...
    'PrintOptions','-dpng' ...
    );

Args=parseArgs(varargin,Args,{'BlackWhite'});


ismetaformat=~isempty(strfind(Args.PrintOptions,'meta'));

%Args.Size=Args.Size*CmPrInch;


sfSet(0,'ShowHiddenHandles','on');

sfSet(f,'Visible','off')
sfSet(f,'paperorientation','portrait');
sfSet(f,'PaperUnits','inches');
sfSet(f,'PaperPositionMode','manual');
sfSet(f,'PaperSize',Args.Size);
sfSet(f,'PaperPosition',[0 0 Args.Size])
%if ismetaformat
sfSet(f,'activepositionproperty','position');
sfSet(f,'Units','inches');
pos=get(f,'position');
sfSet(f,'position',[pos(1:2) Args.Size]);
%end

if Args.BlackWhite
    Cmap=colormap;
    Cmap=mean(Cmap,2); %make colormap b&w
    sfSet(f,'Colormap',[Cmap Cmap Cmap]);
end


%-------------GET LIST OF AXES CHILDREN AND SET AXIS DEFAULT FONT SIZE--------
Hc=get(f,'children');
Hcc=[];
for ii=1:length(Hc)
    if strcmpi(get(Hc(ii),'type'),'axes')&(~strcmpi(get(Hc(ii),'tag'),'legend'))
        sfSet(Hc(ii),'ActivePositionProperty','position');
        sfSet(Hc(ii),'Units','normalized');
        sfSet(Hc(ii),'fontsize',max(get(Hc(ii),'fontsize')*Args.FontSize,Args.MinFontSize));
        HH=get(Hc(ii),'children');
        Hcc(end+(1:length(HH)))=HH(:);
    end
end

%-------------SET CHILD OBJECT PROPERTIES (line size mm.)------------
for ii=1:length(Hcc)
    tp=get(Hcc(ii),'type');
    if strcmp(tp,'patch')
        sfSet(Hcc(ii),'linewidth',max(get(Hcc(ii),'linewidth')*Args.LineWidth,Args.MinLineWidth))
        if Args.BlackWhite
            c=get(Hcc(ii),'FaceVertexCData');
            if length(size(c))==3
                c=mean(c,3);
                c(:,:,[2 3])=c(:,:,[1 1]);
                sfSet(Hcc(ii),'FaceVertexCData',c);
            end
            try
                c=get(Hcc(ii),'FaceColor');
                c=mean(c);
                sfSet(Hcc(ii),'FaceColor',[c c c]);
            catch
                c=get(Hcc(ii),'EdgeColor');
                c=mean(c);
                sfSet(Hcc(ii),'EdgeColor',[c c c]);
            end
        end
    end
    if strcmp(tp,'line')
        sfSet(Hcc(ii),'linewidth',max(get(Hcc(ii),'linewidth')*Args.LineWidth,Args.MinLineWidth))
        sfSet(Hcc(ii),'markersize',max(get(Hcc(ii),'markersize')*Args.MarkerSize,Args.MinMarkerSize))
    end
    if strcmp(tp,'text')
        sfSet(Hcc(ii),'fontunits','points');
        sfSet(Hcc(ii),'fontsize',max(get(Hcc(ii),'fontsize')*Args.FontSize,Args.MinFontSize));
    end
    if strcmp(tp,'image')|strcmp(tp,'patch')
        if Args.BlackWhite
            c=get(Hcc(ii),'CData');
            if length(size(c))==3
                c=mean(im2double(c),3);
                c(:,:,[2 3])=c(:,:,[1 1]);
                sfSet(Hcc(ii),'CData',c);
            end
        end
    end
    if Args.BlackWhite
        try
            c=get(Hcc(ii),'Color');
            c=mean(c);
            sfSet(Hcc(ii),'Color',[c c c]);
        end
    end
end

%-------------CALCULATE CURRENT OUTERBOUNDS FOR AUTOMARGINS-----------
if strcmpi(Args.Margins,'auto')
    Args.Margins=1;
end
if length(Args.Margins)==1; 
    outerbounds=[2 2 -1 -1];
    posbounds=[2 2 -1 -1];
    for ii=1:length(Hc)
        if strcmpi(get(Hc(ii),'type'),'axes')
            ti=get(Hc(ii),'tightinset');
            ps=get(Hc(ii),'position');
            ps(3:4)=ps(1:2)+ps(3:4);
                        

            op=[ps(1:2)-ti(1:2) ps(3:4)+ti(3:4)];
            
            %axes('position',[op(1:2) op(3:4)-op(1:2)],'color','none')
            outerbounds(1:2)=min(outerbounds(1:2),op(1:2));
            outerbounds(3:4)=max(outerbounds(3:4),op(3:4));
            posbounds(1:2)=min(posbounds(1:2),ps(1:2));
            posbounds(3:4)=max(posbounds(3:4),ps(3:4));
        end
    end
    ti=abs(outerbounds-posbounds)*Args.Margins; %effective tightinset
    %Args.Margins=[-outerbounds(1:2) outerbounds(3:4)-1];
    Args.Margins=[0 0 0 0];
    %s=solve('til=pbl*(1-ml-mr)+ml','1-tir=pbr*(1-ml-mr)+ml','ml,mr');
    %s.ml=-(pbl*tir-pbl+pbr*til)/(pbl-pbr)
    %s.mr=(pbl*tir+pbr*til-tir+1-til-pbr)/(pbl-pbr)
    Args.Margins([1 4])=-(posbounds(1:2).*ti(3:4)-posbounds(1:2)+posbounds(3:4).*ti(1:2))./(posbounds(1:2)-posbounds(3:4));
    Args.Margins([2 3])=(posbounds(1:2).*ti(3:4)+posbounds(3:4).*ti(1:2)-ti(3:4)+1-ti(1:2)-posbounds(3:4))./(posbounds(1:2)-posbounds(3:4));
end


%-------------REPOSITION ALL AXES USING MARGINS-----------------------

WH=1-Args.Margins([2 4])-Args.Margins([1 3]); %convert to width,height;
for ii=1:length(Hc)
    if strcmpi(get(Hc(ii),'type'),'axes')&(~strcmpi(get(Hc(ii),'tag'),'legend'))
        newpos=get(Hc(ii),'position');
        newpos([3 4])=newpos([3 4])+newpos([1 2]);
        newpos([1 3])=newpos([1 3])*WH(1)+Args.Margins(1);
        newpos([2 4])=newpos([2 4])*WH(2)+Args.Margins(4);
        newpos([3 4])=newpos([3 4])-newpos([1 2]);
        sfSet(Hc(ii),'ActivePositionProperty','position'); %other wise undo won't work...
        sfSet(Hc(ii),'units','normalized');
        sfSet(Hc(ii),'position',newpos);
    end
end



if ismetaformat
    eval(['print ' Args.PrintOptions ' ' fname] )
else
    eval(['print ' Args.PrintOptions ' -r' num2str(Args.Dpi) ' ' fname] )
end

for ii=length(sf_UndoStack):-1:1
    set(sf_UndoStack{ii}.Handle,sf_UndoStack{ii}.Property,sf_UndoStack{ii}.Value);
end

clear sf_UndoStack

if nargout>=1
    varargout{1}=Args;
end

function sfSet(H,prop,value)
global sf_UndoStack

sf_UndoStack{length(sf_UndoStack)+1}.Handle=H;
sf_UndoStack{length(sf_UndoStack)}.Property=prop;
sf_UndoStack{length(sf_UndoStack)}.Value=get(H,prop);
set(H,prop,value)

% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------

function ArgStruct=parseArgs(args,ArgStruct,varargin)
% Helper function for parsing varargin. 
%
%
% ArgStruct=parseArgs(varargin,ArgStruct[,FlagtypeParams[,Aliases]])
%
% * ArgStruct is the structure full of named arguments with default values.
% * Flagtype params is params that don't require a value. (the value will be set to 1 if it is present)
% * Aliases can be used to map one argument-name to several argstruct fields
%
%
% example usage: 
% --------------
% function parseargtest(varargin)
%
% define the acceptable named arguments and assign default values
% Args=struct('Holdaxis',0, ...
%        'SpacingVertical',0.05,'SpacingHorizontal',0.05, ...
%        'PaddingLeft',0,'PaddingRight',0,'PaddingTop',0,'PaddingBottom',0, ...
%        'MarginLeft',.1,'MarginRight',.1,'MarginTop',.1,'MarginBottom',.1, ...
%        'rows',[],'cols',[]); 
%
% The capital letters define abrreviations.  
%   Eg. parseargtest('spacingvertical',0) is equivalent to  parseargtest('sv',0) 
%
% Args=parseArgs(varargin,Args, ... % fill the arg-struct with values entered by the user
%           {'Holdaxis'}, ... %this argument has no value (flag-type)
%           {'Spacing' {'sh','sv'}; 'Padding' {'pl','pr','pt','pb'}; 'Margin' {'ml','mr','mt','mb'}});
%
% disp(Args)
%
%
%
%
% Aslak Grinsted 2004

% -------------------------------------------------------------------------
%   Copyright (C) 2002-2004, Aslak Grinsted
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made.  This
%   routine is provided as is without any express or implied warranties
%   whatsoever.

persistent matlabver

if isempty(matlabver)
    matlabver=ver('MATLAB');
    matlabver=str2num(matlabver.Version);
end

Aliases={};
FlagTypeParams='';

if (length(varargin)>0) 
    FlagTypeParams=lower(strvcat(varargin{1}));
    if length(varargin)>1
        Aliases=varargin{2};
    end
end
 

%---------------Get "numeric" arguments
NumArgCount=1;
while (NumArgCount<=size(args,2))&(~ischar(args{NumArgCount}))
    NumArgCount=NumArgCount+1;
end
NumArgCount=NumArgCount-1;
if (NumArgCount>0)
    ArgStruct.NumericArguments={args{1:NumArgCount}};
else
    ArgStruct.NumericArguments={};
end 


%--------------Make an accepted fieldname matrix (case insensitive)
Fnames=fieldnames(ArgStruct);
for i=1:length(Fnames)
    name=lower(Fnames{i,1});
    Fnames{i,2}=name; %col2=lower
    AbbrevIdx=find(Fnames{i,1}~=name);
    Fnames{i,3}=[name(AbbrevIdx) ' ']; %col3=abreviation letters (those that are uppercase in the ArgStruct) e.g. SpacingHoriz->sh
    %the space prevents strvcat from removing empty lines
    Fnames{i,4}=isempty(strmatch(Fnames{i,2},FlagTypeParams)); %Does this parameter have a value?
end
FnamesFull=strvcat(Fnames{:,2});
FnamesAbbr=strvcat(Fnames{:,3});

if length(Aliases)>0  
    for i=1:length(Aliases)
        name=lower(Aliases{i,1});
        FieldIdx=strmatch(name,FnamesAbbr,'exact'); %try abbreviations (must be exact)
        if isempty(FieldIdx) 
            FieldIdx=strmatch(name,FnamesFull); %&??????? exact or not? 
        end
        Aliases{i,2}=FieldIdx;
        AbbrevIdx=find(Aliases{i,1}~=name);
        Aliases{i,3}=[name(AbbrevIdx) ' ']; %the space prevents strvcat from removing empty lines
        Aliases{i,1}=name; %dont need the name in uppercase anymore for aliases
    end
    %Append aliases to the end of FnamesFull and FnamesAbbr
    FnamesFull=strvcat(FnamesFull,strvcat(Aliases{:,1})); 
    FnamesAbbr=strvcat(FnamesAbbr,strvcat(Aliases{:,3}));
end

%--------------get parameters--------------------
l=NumArgCount+1; 
while (l<=length(args))
    a=args{l};
    if ischar(a)
        paramHasValue=1; % assume that the parameter has is of type 'param',value
        a=lower(a);
        FieldIdx=strmatch(a,FnamesAbbr,'exact'); %try abbreviations (must be exact)
        if isempty(FieldIdx) 
            FieldIdx=strmatch(a,FnamesFull); 
        end
        if (length(FieldIdx)>1) %shortest fieldname should win 
            [mx,mxi]=max(sum(FnamesFull(FieldIdx,:)==' ',2));
            FieldIdx=FieldIdx(mxi);
        end
        if FieldIdx>length(Fnames) %then it's an alias type.
            FieldIdx=Aliases{FieldIdx-length(Fnames),2}; 
        end
        
        if isempty(FieldIdx) 
            error(['Unknown named parameter: ' a])
        end
        for curField=FieldIdx' %if it is an alias it could be more than one.
            if (Fnames{curField,4})
                if (l+1>length(args))
                    error(['Expected a value for parameter: ' Fnames{curField,1}])
                end
                val=args{l+1};
            else %FLAG PARAMETER
                if (l<length(args)) %there might be a explicitly specified value for the flag
                    val=args{l+1};
                    if isnumeric(val)
                        if (numel(val)==1)
                            val=logical(val);
                        else
                            error(['Invalid value for flag-parameter: ' Fnames{curField,1}])
                        end
                    else
                        val=true;
                        paramHasValue=0; 
                    end
                else
                    val=true;
                    paramHasValue=0; 
                end
            end
            if matlabver>=6
                ArgStruct.(Fnames{curField,1})=val; %try the line below if you get an error here
            else
                ArgStruct=setfield(ArgStruct,Fnames{curField,1},val); %<-works in old matlab versions
            end
        end
        l=l+1+paramHasValue; %if a wildcard matches more than one
    else
        error(['Expected a named parameter: ' num2str(a)])
    end
end