function [h1_out, h2_out] = supertitle(h_fig,str)
% =========================================================================
% --- SUPERTITLE Puts a title above all subplots.
% --- SUPERTITLE('text') adds text to the top of the figure
% --- above all subplots (a "super title"). Use this function
% --- after all subplot commands.
% ---
% --- Drea Thomas 6/15/95 drea@mathworks.com
% --- John Cristion 12/13/00 modified
% --- Mark Histed 03/13/04 histed@mit.edu: fix disappearing legend
% --- 
% --- $Id: SUPERTITLE.m,v 1.2 2004/03/13 22:17:47 histed Exp $
% ---
% --- Warning: If the figure or axis units are non-default, this
% --- will break.
% =========================================================================


% =========================================================================
% --- Parameters used to position the SUPERTITLE.
% =========================================================================
plotregion = 0.960                                                         ; % Amount of the figure window devoted to subplots
titleypos  = 0.970                                                         ; % Y position of title in normalized coordinates
fudge      = 0.025                                                         ; % Fudge factor to adjust y spacing between subplots
fs         = 14                                                            ; % define Fontsize for SUPERTITLE in points
fw         = 'normal'                                                      ; % define Fontweight for SUPERTITLE

% =========================================================================
% --- Store the old current figure & change the current figure 
% =========================================================================
hf_old = get(0,'CurrentFigure')                                            ; % get the handle of the current figure
set(0,'CurrentFigure',h_fig)                                               ; % make 'h_fig' the current fiugre
ha_old = get(h_fig,'CurrentAxes')                                          ; % store the 'current' axes handle

% =========================================================================
% --- Get the bounding rectangle in pixels
% =========================================================================
figunits = get(h_fig,'units')                                              ; % get figure units
set(h_fig,'units','pixels')                                                ; % set figure units property to pixels
pos = get(h_fig,'position')                                                ; % get the figure position in pixels
set(h_fig,'units',figunits)                                                ; % reset the figure units property

% =========================================================================
% --- Get the (approximate) difference between full height
% --- (plot + title + xlabel) and bounding rectangle.
% =========================================================================
ff = (fs-4)*1.27*5/pos(4)*fudge                                            ; % fudge factor
% The 5 here reflects about 3 characters of height below
% an axis and 2 above. 1.27 is pixels per point.

% =========================================================================
% Determine the bounding rectange for all the plots
% and if a title exists
% =========================================================================
h        = findobj(h_fig,'Type','axes')                                    ;  % Change suggested by Stacy J. Hills
max_y    = 0                                                               ;
min_y    = 1                                                               ;
for i=1:length(h)
    pos=get(h(i),'pos')                                                    ;
    if (pos(2) < min_y)
        min_y = pos(2) - 3/5*ff                                            ;
    end
    if (pos(4)+pos(2) > max_y)
        max_y = pos(4) + pos(2) + 2/5*ff                                   ;
    end
end

% =========================================================================
% --- if needed rescale the plotregion
% =========================================================================
if max_y > plotregion,
	scale = (plotregion-min_y)/(max_y-min_y)                               ;
	for i=1:length(h)
		pos    = get(h(i),'position')                                      ;
		pos(2) = (pos(2)-min_y)*scale+min_y                                ;
		pos(4) = pos(4)*scale-(1-scale)*ff/5*3                             ;
		set(h(i),'position',pos)                                           ;
	end
end

% =========================================================================
% --- store the nextplot property and reset it to 'add'
% =========================================================================
np = get(h_fig,'nextplot')                                                 ;
set(h_fig,'nextplot','add')                                                ;

% =========================================================================
% --- create SUPERTITLE & handles to new axes and text
% =========================================================================
ha = axes('pos',[0 1 1 1],'visible','off')                                 ; % create new axes object
ht = text(.5,titleypos-1,str)                                              ; % create text object in the new axes object
set(ht,'HorizontalAlignment','center','Interpreter','none'              ...
      ,'FontWeight',fw,'FontSize',fs)                                      ; % setting for XPCSGUI titles
set(ha,'FontSize',fs,'FontWeight',fw)                                      ; % change 'ha' default font size to 'fs' 

% =========================================================================
% --- reset the nextplot property and make the axes handles current
% =========================================================================
set(h_fig,'nextplot',np)                                                   ;

% =========================================================================
% --- refresh legend if one exists
% =========================================================================
set(h_fig,'CurrentAxes',ha_old)                                            ;
legH = legend                                                              ; % get legend of current axes
if ~isempty(legH)
    axes(legH)                                                             ; % refresh legend
end

% =========================================================================
% --- switch back to previous current figure 
% =========================================================================
set(0,'CurrentFigure',hf_old)                                              ;

% =========================================================================
% --- assign supertitle text handle to h1_out & axes handle to h2_out
% =========================================================================
if ( nargout == 1 )
	h1_out = ht                                                            ;
end
if ( nargout == 2 )
	h1_out = ht                                                            ;
	h2_out = ha                                                            ;
end

% ---
% EOF
